--
-- hello_world.vhd
--
-- The ’Hello World’ example for FPGA programming.
--
-- Author: Martin Schoeberl (martin@jopdesign.com)
--
-- 2006-08-04 created
--
LIBRARY ieee;
    USE ieee.std_logic_1164.ALL;
    USE ieee.numeric_std.ALL;
    
ENTITY hello_world IS
    PORT
    (
        clk     	: IN std_logic;
		button		: IN std_logic;
        led     	: OUT std_logic;
        led2    	: OUT std_logic;
        led3    	: OUT std_logic
    );
END ENTITY hello_world;

ARCHITECTURE rtl OF hello_world IS
	CONSTANT CLK_FREQ   	: integer := 50000000;
	
	SIGNAL blink_freq 		: integer RANGE 0 TO 255 := 51;
	SIGNAL cnt_max 			: unsigned(24 DOWNTO 0) := 25ux"1FFFFFF";
    
	SIGNAL cnt      		: unsigned(24 DOWNTO 0) := (OTHERS => '0');
	SIGNAL blink    		: std_logic := '0';
	SIGNAL blink2   		: std_logic := '0';
	SIGNAL blink3   		: std_logic := '0';
	SIGNAL button_prev		: std_logic := '1';
	SIGNAL button_debounced	: std_logic := '1';
	SIGNAL mult_result		: unsigned(15 DOWNTO 0);
	
	COMPONENT debounce IS
		GENERIC
		(
			counter_size  :  integer := 19		--counter size (19 bits gives 10.5ms with 50MHz clock)
		);
		PORT
		(
			clk     : IN  std_logic;	--input clock
			button  : IN  std_logic;	--input signal to be debounced
			result  : OUT std_logic		--debounced signal
		);
	END COMPONENT debounce;

	COMPONENT multiplier IS
		PORT
		(
			a		: IN unsigned (7 DOWNTO 0);
			b		: IN unsigned (7 DOWNTO 0);
			clk		: IN std_logic;
			aclr	: IN std_logic;
			result	: OUT unsigned (15 DOWNTO 0)
		);
	END COMPONENT multiplier;
	
BEGIN
	i_debounce : debounce
		PORT MAP
		(
			clk => clk,
			button => button,
			result => button_debounced
		);
	
	i_multiplier : multiplier
		PORT MAP
		(
			a => to_unsigned(blink_freq, 8),
			b => x"ff",
			clk => clk,
			aclr => '0',
			result => mult_result
		);
		
	PROCESS(clk)
	BEGIN
		IF rising_edge(clk) THEN
			IF cnt = cnt_max THEN
				cnt <= (OTHERS => '0');
				blink <= not blink;
				blink2 <= not blink2;
			ELSE
				IF cnt = cnt_max / 2 THEN
					blink2 <= not blink2;
					blink3 <= not blink3;
				END IF;
				cnt <= cnt + 1;
			END IF;
			
			IF button_debounced = '0' and button_prev /= '0' THEN
				blink_freq <= blink_freq + 51;

				IF blink_freq = 255 THEN
					blink_freq <= 51;
				END IF;
			END IF;
			button_prev <= button_debounced;
		END IF;
		cnt_max <= mult_result & "000000000";
    END PROCESS;
	led <= blink;
	led2 <= blink2;
	led3 <= blink3;
END rtl;
